var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { either } from 'fp-ts';
import { pipe } from 'fp-ts/lib/function';
import { inject, injectable } from 'inversify';
import * as t from 'io-ts';
import reporter, { formatValidationErrors } from 'io-ts-reporters';
import isPlainObject from 'lodash/isPlainObject';
import { MF_TYPES, isAxiosResponse, plainToNew } from "../..";
let Response = class Response {
    getResponseData(response, typeName) {
        if (!isAxiosResponse(response)) {
            this.logger.toConsole('warn', 'There is no data field in response object');
            return response;
        }
        if (typeName !== undefined) {
            this.checkType(response.data, typeName).forEach(val => this.handleTypeError(val, response.data));
        }
        return response.data;
    }
    getDtoResponse(DTO) {
        return ({ data }) => plainToNew(DTO, data);
    }
    decodeResponseData(type = t.any) {
        return ({ data }) => pipe(type.decode(data), either.mapLeft(formatValidationErrors), either.fold(errors => {
            errors.forEach(val => this.handleTypeError(val, data));
            return data;
        }, t.identity));
    }
    getResponseStatus(response) {
        if ('status' in response) {
            return response.status;
        }
        this.logger.toConsole('warn', 'There is no status field in response object');
        return response;
    }
    getResponseError(error) {
        var _a, _b, _c;
        if (isPlainObject(error) && 'response' in error) {
            return Promise.reject(Error(`${(_a = error === null || error === void 0 ? void 0 : error.response) === null || _a === void 0 ? void 0 : _a.statusText} ${(_b = error === null || error === void 0 ? void 0 : error.response) === null || _b === void 0 ? void 0 : _b.status}` ||
                `Unknown error ${(_c = error === null || error === void 0 ? void 0 : error.response) === null || _c === void 0 ? void 0 : _c.status}`));
        }
        return Promise.reject(error);
    }
    processStatus(status, successMessage, failedMessage = 'An error occurred while executing the request') {
        const success = [200];
        const failure = [404, 403];
        if (success.includes(status)) {
            this.logger.toMessage('info', successMessage);
            return true;
        }
        if (failure.includes(status)) {
            this.logger.toMessage('error', failedMessage);
            return false;
        }
        return undefined;
    }
    checkType(value, type) {
        let report = [];
        if (Array.isArray(value)) {
            value.forEach(x => {
                const validation = type.decode(x);
                report = [...report, ...reporter.report(validation)];
            });
        }
        else {
            const validation = type.decode(value);
            report = reporter.report(validation);
        }
        return report;
    }
    handleTypeError(x, data) {
        this.logger.toError(x, 'Type Error', data);
    }
};
__decorate([
    inject(MF_TYPES.ILogger),
    __metadata("design:type", Object)
], Response.prototype, "logger", void 0);
Response = __decorate([
    injectable()
], Response);
export { Response };
